﻿using Syncfusion.Data;
using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.ScrollAxis;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;

namespace HighlightFilterText
{
    public class SfGridBehavior : Behavior<Window>
    {
        SfDataGrid SfDataGrid;
        Button Button;
        TextBox SearchText;
        static int colIndex = 0, rowIndex = 0, index;
        protected override void OnAttached()
        {
            var window = this.AssociatedObject;
            this.SfDataGrid = window.FindName("datagrid") as SfDataGrid;
            this.Button = window.FindName("serachbutton") as Button;
            this.SearchText = window.FindName("searchTextBox") as TextBox;

            //GridCellTextBoxRenderer is removed.
            this.SfDataGrid.CellRenderers.Remove("TextBox");

            //Customized GridCellTextBoxRenderer is added.
            this.SfDataGrid.CellRenderers.Add("TextBox", new CustomGridCellTextBoxRenderer());
            Button.Click += Button_Click;
        }

        void Button_Click(object sender, RoutedEventArgs e)
        {
            if (this.SfDataGrid.SelectionController.CurrentCellManager.HasCurrentCell)
            {
                colIndex = this.SfDataGrid.SelectionController.CurrentCellManager.CurrentCell.ColumnIndex;
                rowIndex = this.SfDataGrid.SelectionController.CurrentCellManager.CurrentCell.RowIndex;
            }
            else
            {
                if (this.SfDataGrid.View.GroupDescriptions.Count > 0)
                {
                    index = this.SfDataGrid.SelectionController.CurrentCellManager.CurrentRowColumnIndex.RowIndex;
                    FindGroup(index);
                }
                else
                    rowIndex = 0;
                colIndex = this.SfDataGrid.View.GroupDescriptions.Count + 1;
            }

            FindNextCell();
        }
        private void FindGroup(int index)
        {
            var group = this.SfDataGrid.View.TopLevelGroup.DisplayElements[index - 1];
            if (group != null && this.SfDataGrid.View.TopLevelGroup.DisplayElements[index] != null)
            {
                if (this.SfDataGrid.View.TopLevelGroup.DisplayElements[index] is RecordEntry)
                {
                    var recordIndex = this.SfDataGrid.ResolveToRecordIndex(index);
                    rowIndex = this.SfDataGrid.ResolveToRowIndex(recordIndex);
                }
                else
                {
                    index++;
                    FindGroup(index);
                }
            }
            else
                rowIndex = 0;
        }

        private void FindNextCell()
        {
            string searchText = this.SearchText.Text;
            bool isTextHighlight = false;
            var count = this.SfDataGrid.View.GroupDescriptions.Count > 0
                            ? this.SfDataGrid.View.TopLevelGroup.DisplayElements.Count
                            : this.SfDataGrid.View.Records.Count;
            for (int row = rowIndex; row <= count; row++)
            {
                object record = null;

                // searched records when group any column.
                if (this.SfDataGrid.View.GroupDescriptions.Count > 0)
                {
                    if (!(this.SfDataGrid.View.TopLevelGroup.DisplayElements[row - 1] is RecordEntry))
                        continue;
                    var recordIndex = this.SfDataGrid.ResolveToRecordIndex(row - 1);
                    if (recordIndex == this.SfDataGrid.View.Records.Count)
                    {
                        record = this.SfDataGrid.View.Records.GetItemAt(0);
                        row = 0;
                    }
                    else
                        record = this.SfDataGrid.View.Records.GetItemAt(recordIndex);
                    if (this.SfDataGrid.SelectionController.CurrentCellManager.HasCurrentCell)
                        row += 1;
                }
                else // search records when there is no any column grouped.
                {
                    if (row == this.SfDataGrid.View.Records.Count)
                    {
                        record = (this.SfDataGrid.View.Records[0] as RecordEntry).Data;
                        row = 0;
                    }
                    else
                        record = (this.SfDataGrid.View.Records[row] as RecordEntry).Data;

                }
                for (int col = this.SfDataGrid.ResolveToGridVisibleColumnIndex(colIndex); col < this.SfDataGrid.Columns.Count; col++)
                {
                    string value;
                    if (this.SfDataGrid.Columns[col].IsHidden)
                        continue;
                    value =
                        this.SfDataGrid.View.GetPropertyAccessProvider()
                                .GetValue(record, this.SfDataGrid.Columns[col].MappingName)
                                .ToString();
                    if (value.ToLower().Contains(searchText.ToLower()))
                    {
                        isTextHighlight = true;
                        var scrollColIndex = this.SfDataGrid.ResolveToScrollColumnIndex(col);
                        var gridRowIndex = row;
                        if (this.SfDataGrid.View.GroupDescriptions.Count <= 0)
                            gridRowIndex = this.SfDataGrid.ResolveToRowIndex(row);
                        //Move the Selection to next item which is searched.
                        this.SfDataGrid.SelectionController.MoveCurrentCell(new RowColumnIndex(gridRowIndex, scrollColIndex));
                        //scroll the view the next item which is searched.
                        this.SfDataGrid.ScrollInView(new RowColumnIndex(gridRowIndex, scrollColIndex));
                        return;
                    }
                }
                colIndex = this.SfDataGrid.View.GroupDescriptions.Count + 1;
                if (row == count - 1 && !isTextHighlight)
                {
                    MessageBox.Show("Entered text not present in the grid");
                    return;
                }

            }
            rowIndex = 0;
            FindNextCell();
        }
    }
}
